import { EntityEquippableComponent, EntityInventoryComponent, EntityProjectileComponent, EquipmentSlot, GameMode, ItemDurabilityComponent, ItemEnchantableComponent, MinecraftDimensionTypes, system, world } from "@minecraft/server";
import { AmariteDisc, waitTicks } from "./data";
import { TridentManager } from "./manager";

// Add this at the top after imports
const playerThrownCount = new Map();
const discTargetingData = new Map(); // Store targeting information for each disc

world.afterEvents.itemReleaseUse.subscribe((data) => {
    const { source, useDuration } = data;
    
    if (!data.itemStack)
        return;
    
    const mainhand = source.getComponent(EntityEquippableComponent.componentId)?.getEquipmentSlot(EquipmentSlot.Mainhand);
    if (!mainhand)
        return;
    
    const item = mainhand.getItem();
    if (item?.typeId != data.itemStack.typeId)
        return;
    
    const tridentData = AmariteDisc.find((f) => f.itemID == item.typeId);
    if (!tridentData)
        return;
    
    // Check throw limit
    const currentCount = playerThrownCount.get(source.id) || 0;
    if (currentCount >= 3) {
        //source.sendMessage("§cYou can only throw 3 Amarite Discs at a time! Wait for them to return.");
        return;
    }
    
    const enchComp = item.getComponent(ItemEnchantableComponent.componentId);
    
    // Check riptide conditions - only if player has riptide enchantment AND is in water
    if (tridentData.riptide && enchComp?.hasEnchantment("riptide") && source.isInWater) {
        TridentManager.riptide(tridentData, enchComp, source);
        return;
    }
    
    // If we reach here, we should throw the trident
    const durComp = item.getComponent(ItemDurabilityComponent.componentId);
    if (!tridentData.projectile || (durComp && durComp.damage >= durComp.maxDurability))
        return;
    
    const projectileData = tridentData.projectile;
    const eyeLoc = source.getHeadLocation();
    const viewDir = source.getViewDirection();
    
    // Spawn location slightly in front of player
    const spawnLoc = {
        x: eyeLoc.x + viewDir.x * 0.5,
        y: eyeLoc.y + viewDir.y * 0.5,
        z: eyeLoc.z + viewDir.z * 0.5
    };
    
    try {
        console.warn(`Attempting to spawn: ${projectileData.entityID}`);
        const projectile = source.dimension.spawnEntity(projectileData.entityID, spawnLoc);
        console.warn("Entity spawned successfully");
        
        // Increment thrown count after successful spawn
        const newCount = currentCount + 1;
        playerThrownCount.set(source.id, newCount);
        
        // Set disc name based on throw order
        const discNames = ["§dAmarite Disc§r", "§dAmarite Disc§r", "§dAmarite Disc§r"];
        const discName = discNames[newCount - 1];
        projectile.nameTag = discName;
        
        // Set projectile properties
        projectile.setDynamicProperty("item", JSON.stringify(TridentManager.getTridentItem(item)));
        projectile.setDynamicProperty("discName", discName);
        if (enchComp?.getEnchantments()[0])
            projectile.setProperty('amarite_disc_r:enchanted', true);
        projectile.setDynamicProperty("ownerID", source.id);
        
        // Initialize targeting data for this disc
discTargetingData.set(projectile.id, {
    isTargeting: false,
    targetEntity: null,
    stopStartTime: null,
    particleInterval: null,
    chasingTarget: false,
    soundPlayed: false
});
        
        // Rest of your projectile movement code...
        const projectileComp = projectile.getComponent(EntityProjectileComponent.componentId);
        if (projectileComp) {
            console.warn("Using projectile component");
            projectileComp.owner = source;
            
            const shootVelocity = {
                x: viewDir.x * projectileData.thrownVelocity,
                y: viewDir.y * projectileData.thrownVelocity,
                z: viewDir.z * projectileData.thrownVelocity
            };
            
            try {
                projectileComp.shoot(shootVelocity);
                console.warn("Projectile shot with component");
            } catch (e) {
                console.warn(`Projectile shoot failed: ${e}`);
                projectile.applyImpulse(shootVelocity);
                console.warn("Applied impulse instead");
            }
        }
        
        // Play sound
        if (projectileData.thrownSound) {
            source.dimension.playSound(projectileData.thrownSound.id, source.location, { 
                volume: projectileData.thrownSound.volume, 
                pitch: projectileData.thrownSound.pitch 
            });
        }
        
        console.warn(`Trident throw completed - ${discName}`);
        
    } catch (error) {
        console.warn(`Failed to spawn projectile: ${error}`);
    }
});

// Add targeting system for amarite disc rebound
system.runInterval(() => {
    for (const dimension of [world.getDimension("overworld"), world.getDimension("nether"), world.getDimension("the_end")]) {
        const amariteDiscs = dimension.getEntities({ type: "amarite:amarite_disc_thrown_rebound" });
        
        for (const disc of amariteDiscs) {
            if (!disc || !disc.isValid()) continue;
            
            const discId = disc.id;
            const targetingData = discTargetingData.get(discId);
            if (!targetingData) continue;
            
            const ownerID = disc.getDynamicProperty("ownerID");
            if (!ownerID) continue;
            
            const discLocation = disc.location;
            
// If currently chasing a target, continue chasing
if (targetingData.chasingTarget && targetingData.targetEntity && targetingData.targetEntity.isValid()) {
    // Add 1 second delay for the say command
    if (!targetingData.sayCommandExecuted) {
        system.runTimeout(() => {
            if (disc && disc.isValid()) {
                // Sometimes the entity doesn't get hit, so this command fixes that issue
                disc.runCommand("/execute as @e[type=amarite:amarite_disc_thrown_rebound,c=1] at @s run tp @e[type=!amarite:amarite_disc_thrown_rebound,r=1.5] ~~0.001~");
            }
        }, 40); // 20 ticks = 1 second /execute as @e[type=amarite:amarite_disc_thrown_rebound,c=1] at run tp @e[type=!amarite:amarite_disc_thrown_rebound,r=1] ~~0.01~
        targetingData.sayCommandExecuted = true;
    }
    
    const targetLocation = {
        x: targetingData.targetEntity.location.x,
        y: targetingData.targetEntity.location.y + 0.5, // Move target up by half a block
        z: targetingData.targetEntity.location.z
    };
    const distance = Math.sqrt(
        Math.pow(targetLocation.x - discLocation.x, 2) +
        Math.pow(targetLocation.y - discLocation.y, 2) +
        Math.pow(targetLocation.z - discLocation.z, 2)
    );
    
    // Calculate direction to target
    const direction = {
        x: (targetLocation.x - discLocation.x) / distance,
        y: (targetLocation.y - discLocation.y) / distance,
        z: (targetLocation.z - discLocation.z) / distance
    };
    
    // Teleport towards target instead of using impulse (goes through blocks)
    const chaseSpeed = 1.8;
    const newLocation = {
        x: discLocation.x + direction.x * chaseSpeed,
        y: discLocation.y + direction.y * chaseSpeed,
        z: discLocation.z + direction.z * chaseSpeed
    };
    
    disc.teleport(newLocation);
    
    continue;
}

// If currently in targeting mode (stopped and spawning particles)
if (targetingData.isTargeting) {
    // Keep the disc floating in place
    disc.clearVelocity();
    disc.applyImpulse({ x: 0, y: 0.0001, z: 0 }); // Small upward force to counteract gravity
    
    // Make the disc face the target to prevent falling animation
    if (targetingData.targetEntity && targetingData.targetEntity.isValid()) {
        const targetLoc = targetingData.targetEntity.location;
        disc.teleport(discLocation, { facingLocation: targetLoc });
        
        // Play sound only once after 1 second delay
        const currentTime = Date.now();
        const elapsedTime = currentTime - targetingData.stopStartTime;
        
        if (!targetingData.soundPlayed && elapsedTime >= 1000) { // 500 (0.5 seconds)
            disc.runCommand("/playsound Disc_Rebounds @a[r=12] ~~~ 3.2");
            targetingData.soundPlayed = true;
        }
    }
    
    const currentTime = Date.now();
    const elapsedTime = currentTime - targetingData.stopStartTime;
                
                // Check if 2 seconds have passed
                if (elapsedTime >= 2000) {
                    // Stop particles
                    if (targetingData.particleInterval) {
                        system.clearRun(targetingData.particleInterval);
                        targetingData.particleInterval = null;
                    }
                    
                    // Start chasing the target
                    targetingData.isTargeting = false;
                    targetingData.chasingTarget = true;
                    
                    console.warn(`Disc ${disc.nameTag} started chasing target`);
                }
                continue;
            }

// Look for nearby entities (excluding owner and specified types)
const nearbyEntities = dimension.getEntities({
    location: discLocation,
    maxDistance: 15,
    excludeTypes: [
        "minecraft:item", 
        "minecraft:xp_bottle",
        "minecraft:xp_orb",
        "minecraft:arrow",
        "minecraft:fireworks_rocket",
        "minecraft:snowball",
        "minecraft:egg",
        "minecraft:ender_pearl",
        "minecraft:ender_crystal",
        "minecraft:tnt",
        "minecraft:armor_stand",
        "minecraft:breeze_wind_charge_projectile",
        "minecraft:lightning_bolt",
        "minecraft:painting",
        "minecraft:area_effect_cloud",
        "minecraft:dragon_fireball",
        "minecraft:eye_of_ender_signal",
        "minecraft:fireball",
        "minecraft:fishing_hook",
        "minecraft:lingering_potion",
        "minecraft:llama_spit",
        "minecraft:ominous_item_spawner",
        "minecraft:shulker_bullet",
        "minecraft:small_fireball",
        "minecraft:splash_potion",
        "minecraft:thrown_trident",
        "minecraft:tripod_camera",
        "minecraft:wind_charge_projectile",
        "minecraft:wither_skull",
        "minecraft:wither_skull_dangerous",
        "minecraft:chest_boat",
        "minecraft:boat",

        // Custom Entities
        "amarite:amarite_disc_thrown",
        "amarite:amarite_disc_thrown_trichotomy",
        "amarite:amarite_disc_thrown_rebound",

        // Impaled Tridents
        "custom_trident:water_checker",
        "custom_trident:atlan_thrown_trident",
        "custom_trident:thrown_trident",
        "custom_trident:elder_thrown_trident",
        "custom_trident:enderite_thrown_trident",
        "custom_trident:frostbite_thrown_trident",
        "custom_trident:guardian_thrown_trident",
        "custom_trident:hellfork_thrown_trident",
        "custom_trident:my_thrown_trident",
        "custom_trident:pitchfork_thrown_trident",
        "custom_trident:radiation_thrown_trident",
        "custom_trident:soulfork_thrown_trident",
        "custom_trident:titanium_thrown_trident",
    ]
});

let targetFound = false;
for (const entity of nearbyEntities) {
    if (!entity || !entity.isValid()) continue;
    if (entity.id === ownerID) continue; // Skip owner
    if (entity.id === discId) continue; // Skip self
    
    // Check for target:false tag
    if (entity.hasTag("has_target:false")) continue;
    
    // Found a valid target
    targetingData.isTargeting = true;
    targetingData.targetEntity = entity;
    targetingData.stopStartTime = Date.now();
                
                // Stop the disc's movement
                disc.clearVelocity();
                
                // Start spawning particles around the disc
                targetingData.particleInterval = system.runInterval(() => {
                    if (!disc || !disc.isValid()) return;
                    
                    const loc = disc.location;
                    // Spawn particles in a circle around the disc
                    for (let i = 0; i < 8; i++) {
                        const angle = (i / 8) * Math.PI * 2;
                        const radius = 1;
                        const particleLoc = {
                            x: loc.x + Math.cos(angle) * radius,
                            y: loc.y + Math.sin(i * 0.5) * 0.5,
                            z: loc.z + Math.sin(angle) * radius
                        };
                        
                        try {
                            dimension.spawnParticle("minecraft:endrod", particleLoc);
                        } catch (e) {
                            // Particle spawn failed, continue
                        }
                    }
                }, 2); // Spawn particles every 2 ticks
                
                console.warn(`Disc ${disc.nameTag} found target and stopped moving`);
                targetFound = true;
                break;
            }
        }
    }
}, 1); // Run every tick for responsive targeting

// Add debugging for item use start
world.afterEvents.itemStartUse.subscribe((data) => {
    const { source, itemStack } = data;
    if (itemStack && itemStack.typeId === "amarite:amarite_disc_rebound") {
        console.warn(`Item start use: ${itemStack.typeId} by ${source.name}`);
    }
});

// Add debugging for item use
world.afterEvents.itemUse.subscribe((data) => {
    const { source, itemStack } = data;
    if (itemStack && itemStack.typeId === "amarite:amarite_disc_rebound") {
        console.warn(`Item use: ${itemStack.typeId} by ${source.name}`);
    }
});


// Rest of your code remains the same...
world.afterEvents.projectileHitBlock.subscribe((data) => {
    const { projectile } = data;
    if (!projectile || !projectile.isValid())
        return;
    let itemData = projectile.getDynamicProperty("item");
    if (!itemData)
        return;
    itemData = JSON.parse(itemData);
    
    // Clean up targeting data when projectile hits block
    const targetingData = discTargetingData.get(projectile.id);
    if (targetingData && targetingData.particleInterval) {
        system.clearRun(targetingData.particleInterval);
    }
    discTargetingData.delete(projectile.id);
    
    // Always trigger returning behavior (removed loyalty requirement)
    system.runTimeout(() => { 
        if (projectile && projectile.isValid())
            projectile.triggerEvent("amarite_disc_r:returning");
            projectile.runCommand("/event entity @s amarite_disc_r:returning"); // faster return
            projectile.runCommand("/playsound Disc_hits_Block @a[r=8] ~~~ 3.2");
    });
    
    // Removed channeling lightning rod functionality
});

world.afterEvents.projectileHitEntity.subscribe((data) => {
    const { projectile } = data;
    if (!projectile || !projectile.isValid())
        return;
    let itemData = projectile.getDynamicProperty("item");
    if (!itemData)
        return;
    itemData = JSON.parse(itemData);
    if (TridentManager.reduceDurability(itemData) && itemData.durabilityDamage !== undefined) {
        itemData.durabilityDamage += 1;
        projectile.setDynamicProperty("item", JSON.stringify(itemData));
    }
    
    const hitEntity = data.getEntityHit().entity;
    
    // Clean up targeting data when projectile hits entity
    const targetingData = discTargetingData.get(projectile.id);
    if (targetingData && targetingData.particleInterval) {
        system.clearRun(targetingData.particleInterval);
    }
    discTargetingData.delete(projectile.id);
    
    // Always trigger returning behavior (removed loyalty requirement)
    system.runTimeout(() => { 
        if (projectile && projectile.isValid())
            projectile.triggerEvent("amarite_disc_r:returning"); 
            projectile.runCommand("/event entity @s amarite_disc_r:returning"); // faster return
            projectile.runCommand("/playsound Disc_hits_Entity @a[r=8] ~~~ 3.2");
    }, waitTicks);
    
    // Removed channeling lightning functionality
});


system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
        if (!player || !player.isValid())
            continue;
        const { x, y, z } = player.location;
        const tridents = player.dimension.getEntities({ location: { x: x, y: y + 1, z: z }, maxDistance: 2.2, excludeTypes: ["minecraft:player", "minecraft:item", "minecraft:zombie", "minecraft:skeleton", "minecraft:chicken"] });
        const inv = player.getComponent(EntityInventoryComponent.componentId);
        if (!inv.container || inv.container.emptySlotsCount == 0)
            continue;
        const container = inv.container;
        for (let i = 0; i < tridents.length; i++) {
            if (!tridents[i] || !tridents[i].isValid())
                continue;
            if (container.emptySlotsCount <= 0)
                continue;
            const found = AmariteDisc.find((f) => f.projectile?.entityID == tridents[i].typeId);
            if (!found)
                continue;
            const tridentEntity = tridents[i];
            if (!TridentManager.canPickUp(tridentEntity))
                continue;
            const ownerID = tridentEntity.getDynamicProperty("ownerID");
            if (!ownerID)
                continue;
            if (ownerID != player.id)
                continue;
            const itemData = tridentEntity.getDynamicProperty("item");
            if (!itemData)
                continue;
            const gameMode = player.getGameMode();
            if (gameMode != GameMode.creative && gameMode != GameMode.spectator && gameMode != GameMode.survival) {
                const item = TridentManager.getItem(JSON.parse(itemData));
                container.addItem(item);
            }
            
            // Clean up targeting data when disc is picked up
            const targetingData = discTargetingData.get(tridentEntity.id);
            if (targetingData && targetingData.particleInterval) {
                system.clearRun(targetingData.particleInterval);
            }
            discTargetingData.delete(tridentEntity.id);
            
            // Decrement thrown count when disc is actually picked up
            const currentCount = playerThrownCount.get(player.id) || 0;
            if (currentCount > 0) {
                playerThrownCount.set(player.id, currentCount - 1);
            }
            
            // Log which disc was picked up
            const discName = tridentEntity.getDynamicProperty("discName") || "unknown";
            console.warn(`${discName} picked up by ${player.name}`);
            
            player.dimension.playSound("Disc_Picked_Up", tridentEntity.location, { pitch: 1 + Math.random(), volume: 5.5 });
            tridentEntity.remove();
        }
    }
}, 3);

system.afterEvents.scriptEventReceive.subscribe((data) => {
    if (data.id != "amarite_r:amarite_disc_return" && data.id != "amarite_r:amarite_disc_tick")
        return;
    const tridentEntity = data.sourceEntity;
    if (!tridentEntity || !tridentEntity.isValid())
        return;
    const tridentData = AmariteDisc.find((f) => f.projectile?.entityID == tridentEntity.typeId);
    if (!tridentData)
        return;
    if (data.id == "amarite_r:amarite_disc_return") {
        let itemData = tridentEntity.getDynamicProperty("item");
        if (!itemData)
            return;
        itemData = JSON.parse(itemData);
        
        // Clean up targeting data when disc returns
        const targetingData = discTargetingData.get(tridentEntity.id);
        if (targetingData && targetingData.particleInterval) {
            system.clearRun(targetingData.particleInterval);
        }
        discTargetingData.delete(tridentEntity.id);
        
        const ownerID = tridentEntity.getDynamicProperty("ownerID");
        if (!ownerID)
            return;
        const owner = TridentManager.getOwner(ownerID);
        if (!owner)
            return;
        if (!tridentEntity || !tridentEntity.isValid() || !owner || !owner.isValid())
            return;
        const loc = tridentEntity.location;
        let velocity = tridentData.projectile?.returnSpeed;
        if (velocity === undefined)
            velocity = 1;
        
        // Removed loyalty level multiplier, using base velocity
        const ownerLoc = owner.location;
        tridentEntity.teleport(loc, { facingLocation: { x: ownerLoc.x, y: ownerLoc.y + 1, z: ownerLoc.z } });
        const viewDir = tridentEntity.getViewDirection();
        tridentEntity.teleport({ x: loc.x + viewDir.x + (viewDir.x * velocity), y: loc.y + viewDir.y + (viewDir.y * velocity), z: loc.z + (viewDir.z * velocity) });
        {
            if (tridentEntity.getDynamicProperty("returning"))
                return;
            tridentEntity.setDynamicProperty("returning", true);
            if (tridentData.projectile?.onReturn)
                tridentData.projectile.onReturn(tridentEntity, tridentEntity.dimension, owner, 1); // Default level 1 instead of loyalty.lvl
            if (tridentData.projectile?.returnSound) {
                const sound = tridentData.projectile.returnSound;
                if (owner.typeId == "minecraft:player")
                    owner.playSound(sound.id, { volume: sound.volume, pitch: sound.pitch });
            }
        }
    }
    else if (tridentEntity.location.y < -64) {
        tridentEntity.runCommand("scriptevent amarite_r:amarite_disc_return");
    }
    return;
});

// Add the actionbar display code here, after all other code
system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
        if (!player || !player.isValid())
            continue;
            
        const mainhand = player.getComponent(EntityEquippableComponent.componentId)?.getEquipmentSlot(EquipmentSlot.Mainhand);
        if (!mainhand)
            continue;
            
        const item = mainhand.getItem();
        if (item?.typeId !== "amarite:amarite_disc_rebound")
            continue;
            
        const currentCount = playerThrownCount.get(player.id) || 0;
        const remainingThrows = 3 - currentCount;
        
        let display = "";
        if (remainingThrows === 0) {
            display = "";
        } else if (remainingThrows === 3) {
            display = "";
        } else if (remainingThrows === 2) {
            display = "";
        } else if (remainingThrows === 1) {
            display = "";
        }
        
        player.onScreenDisplay.setActionBar(`§d${display}§r`);
    }
}, 5);
