import { EntityEquippableComponent, EntityInventoryComponent, EntityProjectileComponent, EquipmentSlot, GameMode, ItemDurabilityComponent, ItemEnchantableComponent, MinecraftDimensionTypes, system, world } from "@minecraft/server";
import { AmariteDisc, waitTicks } from "./data";
import { TridentManager } from "./manager";

// Add this at the top after imports
const playerThrownCount = new Map();

world.afterEvents.itemReleaseUse.subscribe((data) => {
    const { source, useDuration } = data;
    
    if (!data.itemStack)
        return;
    
    const mainhand = source.getComponent(EntityEquippableComponent.componentId)?.getEquipmentSlot(EquipmentSlot.Mainhand);
    if (!mainhand)
        return;
    
    const item = mainhand.getItem();
    if (item?.typeId != data.itemStack.typeId)
        return;
    
    const tridentData = AmariteDisc.find((f) => f.itemID == item.typeId);
    if (!tridentData)
        return;
    
    // Check throw limit
    const currentCount = playerThrownCount.get(source.id) || 0;
    if (currentCount >= 3) {
        //source.sendMessage("§cYou can only throw 3 Amarite Discs at a time! Wait for them to return.");
        return;
    }
    
    const enchComp = item.getComponent(ItemEnchantableComponent.componentId);
    
    // Check riptide conditions - only if player has riptide enchantment AND is in water
    if (tridentData.riptide && enchComp?.hasEnchantment("riptide") && source.isInWater) {
        TridentManager.riptide(tridentData, enchComp, source);
        return;
    }
    
    // If we reach here, we should throw the trident
    const durComp = item.getComponent(ItemDurabilityComponent.componentId);
    if (!tridentData.projectile || (durComp && durComp.damage >= durComp.maxDurability))
        return;
    
    const projectileData = tridentData.projectile;
    const eyeLoc = source.getHeadLocation();
    const viewDir = source.getViewDirection();
    
    // Spawn location slightly in front of player
    const spawnLoc = {
        x: eyeLoc.x + viewDir.x * 0.5,
        y: eyeLoc.y + viewDir.y * 0.5,
        z: eyeLoc.z + viewDir.z * 0.5
    };
    
    try {
        console.warn(`Attempting to spawn: ${projectileData.entityID}`);
        const projectile = source.dimension.spawnEntity(projectileData.entityID, spawnLoc);
        console.warn("Entity spawned successfully");
        
        // Increment thrown count after successful spawn
        const newCount = currentCount + 1;
        playerThrownCount.set(source.id, newCount);
        
        // Set disc name based on throw order
        const discNames = ["§dAmarite Disc§r", "§dAmarite Disc§r", "§dAmarite Disc§r"];
        const discName = discNames[newCount - 1];
        projectile.nameTag = discName;
        
        // Set projectile properties
        projectile.setDynamicProperty("item", JSON.stringify(TridentManager.getTridentItem(item)));
        projectile.setDynamicProperty("discName", discName);
        if (enchComp?.getEnchantments()[0])
            projectile.setProperty('amarite_disc:enchanted', true);
        projectile.setDynamicProperty("ownerID", source.id);
        
        // Rest of your projectile movement code...
        const projectileComp = projectile.getComponent(EntityProjectileComponent.componentId);
        if (projectileComp) {
            console.warn("Using projectile component");
            projectileComp.owner = source;
            
            const shootVelocity = {
                x: viewDir.x * projectileData.thrownVelocity,
                y: viewDir.y * projectileData.thrownVelocity,
                z: viewDir.z * projectileData.thrownVelocity
            };
            
            try {
                projectileComp.shoot(shootVelocity);
                console.warn("Projectile shot with component");
            } catch (e) {
                console.warn(`Projectile shoot failed: ${e}`);
                projectile.applyImpulse(shootVelocity);
                console.warn("Applied impulse instead");
            }
        }
        
        // Play sound
        if (projectileData.thrownSound) {
            source.dimension.playSound(projectileData.thrownSound.id, source.location, { 
                volume: projectileData.thrownSound.volume, 
                pitch: projectileData.thrownSound.pitch 
            });
        }
        
        console.warn(`Trident throw completed - ${discName}`);
        
    } catch (error) {
        console.warn(`Failed to spawn projectile: ${error}`);
    }
});

// Add debugging for item use start
world.afterEvents.itemStartUse.subscribe((data) => {
    const { source, itemStack } = data;
    if (itemStack && itemStack.typeId === "amarite:amarite_disc") {
        console.warn(`Item start use: ${itemStack.typeId} by ${source.name}`);
    }
});

// Add debugging for item use
world.afterEvents.itemUse.subscribe((data) => {
    const { source, itemStack } = data;
    if (itemStack && itemStack.typeId === "amarite:amarite_disc") {
        console.warn(`Item use: ${itemStack.typeId} by ${source.name}`);
    }
});

// Rest of your code remains the same...
world.afterEvents.projectileHitBlock.subscribe((data) => {
    const { projectile } = data;
    if (!projectile || !projectile.isValid())
        return;
    let itemData = projectile.getDynamicProperty("item");
    if (!itemData)
        return;
    itemData = JSON.parse(itemData);
    
    // Always trigger returning behavior (removed loyalty requirement)
    system.runTimeout(() => { 
        if (projectile && projectile.isValid())
            projectile.triggerEvent("amarite_disc:returning");
            projectile.runCommand("/event entity @s amarite_disc:returning"); // faster return
            projectile.runCommand("/playsound Disc_hits_Block @a[r=8] ~~~ 3.2");
    });
    
    // Removed channeling lightning rod functionality
});

world.afterEvents.projectileHitEntity.subscribe((data) => {
    const { projectile } = data;
    if (!projectile || !projectile.isValid())
        return;
    let itemData = projectile.getDynamicProperty("item");
    if (!itemData)
        return;
    itemData = JSON.parse(itemData);
    if (TridentManager.reduceDurability(itemData) && itemData.durabilityDamage !== undefined) {
        itemData.durabilityDamage += 1;
        projectile.setDynamicProperty("item", JSON.stringify(itemData));
    }
    
    const hitEntity = data.getEntityHit().entity;
    
    // Always trigger returning behavior (removed loyalty requirement)
    system.runTimeout(() => { 
        if (projectile && projectile.isValid())
            projectile.triggerEvent("amarite_disc:returning"); 
            projectile.runCommand("/event entity @s amarite_disc:returning"); // faster return
            projectile.runCommand("/playsound Disc_hits_Entity @a[r=8] ~~~ 3.2");
    }, waitTicks);
    
    // Removed channeling lightning functionality
});

system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
        if (!player || !player.isValid())
            continue;
        const { x, y, z } = player.location;
        const tridents = player.dimension.getEntities({ location: { x: x, y: y + 1, z: z }, maxDistance: 2.2, excludeTypes: ["minecraft:player", "minecraft:item", "minecraft:zombie", "minecraft:skeleton", "minecraft:chicken"] });
        const inv = player.getComponent(EntityInventoryComponent.componentId);
        if (!inv.container || inv.container.emptySlotsCount == 0)
            continue;
        const container = inv.container;
        for (let i = 0; i < tridents.length; i++) {
            if (!tridents[i] || !tridents[i].isValid())
                continue;
            if (container.emptySlotsCount <= 0)
                continue;
            const found = AmariteDisc.find((f) => f.projectile?.entityID == tridents[i].typeId);
            if (!found)
                continue;
            const tridentEntity = tridents[i];
            if (!TridentManager.canPickUp(tridentEntity))
                continue;
            const ownerID = tridentEntity.getDynamicProperty("ownerID");
            if (!ownerID)
                continue;
            if (ownerID != player.id)
                continue;
            const itemData = tridentEntity.getDynamicProperty("item");
            if (!itemData)
                continue;
            const gameMode = player.getGameMode();
            if (gameMode != GameMode.creative && gameMode != GameMode.spectator && gameMode != GameMode.survival) {
                const item = TridentManager.getItem(JSON.parse(itemData));
                container.addItem(item);
            }
            
            // Decrement thrown count when disc is actually picked up
            const currentCount = playerThrownCount.get(player.id) || 0;
            if (currentCount > 0) {
                playerThrownCount.set(player.id, currentCount - 1);
            }
            
            // Log which disc was picked up
            const discName = tridentEntity.getDynamicProperty("discName") || "unknown";
            console.warn(`${discName} picked up by ${player.name}`);
            
            player.dimension.playSound("Disc_Picked_Up", tridentEntity.location, { pitch: 1 + Math.random(), volume: 5.5 });
            tridentEntity.remove();
        }
    }
}, 3);

system.afterEvents.scriptEventReceive.subscribe((data) => {
    if (data.id != "amarite:amarite_disc_return" && data.id != "amarite:amarite_disc_tick")
        return;
    const tridentEntity = data.sourceEntity;
    if (!tridentEntity || !tridentEntity.isValid())
        return;
    const tridentData = AmariteDisc.find((f) => f.projectile?.entityID == tridentEntity.typeId);
    if (!tridentData)
        return;
    if (data.id == "amarite:amarite_disc_return") {
        let itemData = tridentEntity.getDynamicProperty("item");
        if (!itemData)
            return;
        itemData = JSON.parse(itemData);
        
        const ownerID = tridentEntity.getDynamicProperty("ownerID");
        if (!ownerID)
            return;
        const owner = TridentManager.getOwner(ownerID);
        if (!owner)
            return;
        if (!tridentEntity || !tridentEntity.isValid() || !owner || !owner.isValid())
            return;
        const loc = tridentEntity.location;
        let velocity = tridentData.projectile?.returnSpeed;
        if (velocity === undefined)
            velocity = 1;
        
        // Removed loyalty level multiplier, using base velocity
        const ownerLoc = owner.location;
        tridentEntity.teleport(loc, { facingLocation: { x: ownerLoc.x, y: ownerLoc.y + 1, z: ownerLoc.z } });
        const viewDir = tridentEntity.getViewDirection();
        tridentEntity.teleport({ x: loc.x + viewDir.x + (viewDir.x * velocity), y: loc.y + viewDir.y + (viewDir.y * velocity), z: loc.z + (viewDir.z * velocity) });
        {
            if (tridentEntity.getDynamicProperty("returning"))
                return;
            tridentEntity.setDynamicProperty("returning", true);
            if (tridentData.projectile?.onReturn)
                tridentData.projectile.onReturn(tridentEntity, tridentEntity.dimension, owner, 1); // Default level 1 instead of loyalty.lvl
            if (tridentData.projectile?.returnSound) {
                const sound = tridentData.projectile.returnSound;
                if (owner.typeId == "minecraft:player")
                    owner.playSound(sound.id, { volume: sound.volume, pitch: sound.pitch });
            }
        }
    }
    else if (tridentEntity.location.y < -64) {
        tridentEntity.runCommand("scriptevent amarite:amarite_disc_return");
    }
    return;
});

// Add the actionbar display code here, after all other code
system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
        if (!player || !player.isValid())
            continue;
            
        const mainhand = player.getComponent(EntityEquippableComponent.componentId)?.getEquipmentSlot(EquipmentSlot.Mainhand);
        if (!mainhand)
            continue;
            
        const item = mainhand.getItem();
        if (item?.typeId !== "amarite:amarite_disc")
            continue;
            
        const currentCount = playerThrownCount.get(player.id) || 0;
        const remainingThrows = 3 - currentCount;
        
        let display = "";
        if (remainingThrows === 0) {
            display = "";
        } else if (remainingThrows === 3) {
            display = "";
        } else if (remainingThrows === 2) {
            display = "";
        } else if (remainingThrows === 1) {
            display = "";
        }
        
        player.onScreenDisplay.setActionBar(`§d${display}§r`);
    }
}, 5);
