import { world, system, ItemStack } from '@minecraft/server';

// Map beacon effects to mirror variants
const BEACON_EFFECT_TO_MIRROR = {
    'haste': 'amarite:amarite_mirror=haste',
    'speed': 'amarite:amarite_mirror=speed',
    'jump_boost': 'amarite:amarite_mirror=jump_boost',
    'strength': 'amarite:amarite_mirror=strength',
    'regeneration': 'amarite:amarite_mirror=regeneration',
    'resistance': 'amarite:amarite_mirror=resistance'
};

// Listen for item use on block events
world.beforeEvents.itemUseOn.subscribe((eventData) => {
    const { source: player, itemStack, block } = eventData;
    
    // Check if player is sneaking and using amarite mirror
    if (!player.isSneaking || !itemStack || itemStack.typeId !== 'amarite:amarite_mirror') {
        return;
    }
    
    // Check if the block is a beacon or conduit
    if (block.typeId === 'minecraft:beacon') {
        // Cancel the default interaction
        eventData.cancel = true;
        
        // Schedule the mirror transformation for next tick
        system.run(() => {
            transformMirror(player, block);
        });
    } else if (block.typeId === 'minecraft:conduit') {
        // Cancel the default interaction
        eventData.cancel = true;
        
        // Schedule the conduit mirror transformation for next tick
        system.run(() => {
            transformMirrorWithConduit(player, block);
        });
    }
});

function transformMirror(player, beaconBlock) {
    try {
        // Get the beacon's current effect
        const beaconEffect = getBeaconEffect(beaconBlock);
        
        if (!beaconEffect) {
            player.sendMessage("§cThis beacon doesn't have an active effect!");
            return;
        }
        
        // Get the corresponding mirror variant
        const mirrorVariant = BEACON_EFFECT_TO_MIRROR[beaconEffect];
        
        if (!mirrorVariant) {
            player.sendMessage(`§cNo mirror variant found for effect: ${beaconEffect}`);
            return;
        }
        
        // Get player's inventory and find the amarite mirror
        const inventory = player.getComponent('minecraft:inventory').container;
        const selectedSlot = player.selectedSlotIndex;
        const currentItem = inventory.getItem(selectedSlot);
        
        if (!currentItem || currentItem.typeId !== 'amarite:amarite_mirror') {
            return;
        }
        
        // Create the new mirror variant
        const newMirror = new ItemStack(mirrorVariant, 1);
        
        // Copy any existing enchantments or data from the original mirror
        if (currentItem.hasComponent('minecraft:enchantable')) {
            const enchantments = currentItem.getComponent('minecraft:enchantable');
            if (newMirror.hasComponent('minecraft:enchantable')) {
                // Copy enchantments if both items support them
                const newEnchantments = newMirror.getComponent('minecraft:enchantable');
                // Note: Enchantment copying would need specific implementation based on your needs
            }
        }
        
        // Replace the item in the player's hand
        inventory.setItem(selectedSlot, newMirror);
        
        // Send success message
        const effectName = beaconEffect.replace('_', ' ').replace(/\b\w/g, l => l.toUpperCase());
        player.sendMessage(`§aAmarite Mirror transformed! Now attuned to: §e${effectName}`);
        
        // Play transformation effects
        playTransformationEffects(player, beaconBlock);
        
    } catch (error) {
        console.warn('Error transforming amarite mirror:', error);
        player.sendMessage("§cFailed to transform the Amarite Mirror!");
    }
}

function transformMirrorWithConduit(player, conduitBlock) {
    try {
        // Check if player has conduit_power effect
        const conduitPowerEffect = player.getEffect('conduit_power');
        
        if (!conduitPowerEffect) {
            player.sendMessage("§cYou need to have Conduit Power to attune the mirror to this conduit!");
            return;
        }
        
        // Get player's inventory and find the amarite mirror
        const inventory = player.getComponent('minecraft:inventory').container;
        const selectedSlot = player.selectedSlotIndex;
        const currentItem = inventory.getItem(selectedSlot);
        
        if (!currentItem || currentItem.typeId !== 'amarite:amarite_mirror') {
            return;
        }
        
        // Create the new conduit mirror variant
        const newMirror = new ItemStack('amarite:amarite_mirror=conduit', 1);
        
        // Copy any existing enchantments or data from the original mirror
        if (currentItem.hasComponent('minecraft:enchantable')) {
            const enchantments = currentItem.getComponent('minecraft:enchantable');
            if (newMirror.hasComponent('minecraft:enchantable')) {
                // Copy enchantments if both items support them
                const newEnchantments = newMirror.getComponent('minecraft:enchantable');
                // Note: Enchantment copying would need specific implementation based on your needs
            }
        }
        
        // Replace the item in the player's hand
        inventory.setItem(selectedSlot, newMirror);
        
        // Send success message
        player.sendMessage(`§aAmarite Mirror transformed! Now attuned to: §bConduit Power`);
        
        // Play transformation effects
        playTransformationEffects(player, conduitBlock);
        
    } catch (error) {
        console.warn('Error transforming amarite mirror with conduit:', error);
        player.sendMessage("§cFailed to transform the Amarite Mirror!");
    }
}

function getBeaconEffect(beaconBlock) {
    try {
        // Check if beacon has an active effect
        // Note: This is a simplified approach - you may need to check the beacon's
        // actual pyramid structure and stored effect data
        
        // Get entities near the beacon to check for active effects
        const nearbyEntities = beaconBlock.dimension.getEntitiesAtBlockLocation(beaconBlock.location);
        
        // Alternative approach: Check for beacon effects in the area
        const testLocation = {
            x: beaconBlock.location.x,
            y: beaconBlock.location.y + 1,
            z: beaconBlock.location.z
        };
        
        // Try to detect beacon effects by checking common beacon effect types
        // This is a workaround since direct beacon data access might be limited
        const possibleEffects = ['haste', 'speed', 'jump_boost', 'strength', 'regeneration', 'resistance'];
        
        // Check players in beacon range for active effects that match beacon effects
        const playersInRange = beaconBlock.dimension.getPlayers({
            location: beaconBlock.location,
            maxDistance: 50 // Beacon range varies by pyramid level
        });
        
        for (const player of playersInRange) {
            for (const effectType of possibleEffects) {
                if (player.getEffect(effectType)) {
                    // Found a player with a beacon effect - assume this is the beacon's effect
                    return effectType;
                }
            }
        }
        
        // If no effects detected, return null
        return null;
        
    } catch (error) {
        console.warn('Error getting beacon effect:', error);
        return null;
    }
}

function playTransformationEffects(player, block) {
    try {
        // Play sound effect
        player.playSound('random.levelup', {
            location: player.location,
            volume: 1.0,
            pitch: 1.2
        });
        
        // Spawn particles at block location
        const particleLocation = {
            x: block.location.x + 0.5,
            y: block.location.y + 1,
            z: block.location.z + 0.5
        };
        
        block.dimension.spawnParticle('minecraft:enchanting_table_particle', particleLocation);
        
        // Add brief glowing effect to player
        player.addEffect('glowing', 40, {
            amplifier: 0,
            showParticles: true
        });
        
    } catch (error) {
        console.warn('Error playing transformation effects:', error);
    }
}

// Optional: Add a command to manually check beacon effects for debugging
world.beforeEvents.chatSend.subscribe((eventData) => {
    const { sender: player, message } = eventData;
    
    if (message === '!checkbeacon') {
        eventData.cancel = true;
        
        system.run(() => {
            const block = player.getBlockFromViewDirection({ maxDistance: 10 });
            
            if (block && block.block.typeId === 'minecraft:beacon') {
                const effect = getBeaconEffect(block.block);
                player.sendMessage(`§aBeacon effect: §e${effect || 'None detected'}`);
            } else {
                player.sendMessage('§cNo beacon found in view direction!');
            }
        });
    }
    
    if (message === '!checkconduit') {
        eventData.cancel = true;
        
        system.run(() => {
            const conduitPower = player.getEffect('conduit_power');
            player.sendMessage(`§aConduit Power effect: §e${conduitPower ? 'Active' : 'Not active'}`);
        });
    }
});
